/*
 * Compatibility functions
 *
 * Copyright 2002 by Gray Watson
 *
 * This file is part of the diskheap package.
 *
 * Permission to use, copy, modify, and distribute this software for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of Gray Watson not be used in advertising
 * or publicity pertaining to distribution of the document or software
 * without specific, written prior permission.
 *
 * Gray Watson makes no representations about the suitability of the
 * software described herein for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * The author may be contacted via http://256.com/gray/
 *
 * $Id: compat.c,v 1.1 2002/02/14 23:58:11 gray Exp $
 */

#include "conf.h"

/*
 * This file holds the compatibility routines necessary for the library to
 * function just in case your system does not have them.
 */

#ifdef DMALLOC
#include "dmalloc.h"
#endif

#if IDENT_WORKS
#ident "$Id: compat.c,v 1.1 2002/02/14 23:58:11 gray Exp $"
#else
static char *rcs_id = "$Id: compat.c,v 1.1 2002/02/14 23:58:11 gray Exp $";
#endif

#if HAVE_MEMMOVE == 0
/*
 * Copy LEN characters from SRC to DEST
 */
void	*memmove(void *dest, const void *src, const DMALLOC_SIZE len)
{
  unsigned char		*dest_p;
  const	unsigned char	*src_p;
  int			byte_c;
  
  if (len <= 0) {
    return;
  }
  
  src_p = src;
  dest_p = dest;
  
  if (src_p <= dest_p && src_p + (len - 1) >= dest_p) {
    /* overlap, must copy right-to-left. */
    src_p += len - 1;
    dest_p += len - 1;
    for (byte_c = 0; byte_c < len; byte_c++) {
      *dest_p-- = *src_p--;
    }
  } else {
    for (byte_c = 0; byte_c < len; byte_c++) {
      *dest_p++ = *src_p++;
    }
  }
  
  return dest;
}
#endif /* HAVE_MEMMOVE == 0 */

#if HAVE_MEMCMP == 0
/*
 * Compare LEN characters, return -1,0,1 if STR1 is <,==,> STR2
 */
int	memcmp(const void *str1, const void *str2, DMALLOC_SIZE len)
{
  const unsigned char	*str1_p = str1, *str2_p = str2;
  int			byte_c;
  
  for (byte_c = 0; byte_c < len; byte_c++) {
    if (*str1_p != *str2_p) {
      return *str1_p - *str2_p;
    }
    str1_p++;
    str2_p++;
  }
  
  return 0;
}
#endif /* HAVE_MEMCMP == 0 */

#if HAVE_MEMSET == 0
/*
 * Set LEN characters in STR to character CH
 */
void	*memset(void *str, const int ch, const DMALLOC_SIZE len)
{
  unsigned char	*str_p = str;
  int		byte_c;
  
  for (byte_c = 0; byte_c < len; byte_c++) {
    *str_p++ = ch;
  }
  
  return str;
}
#endif /* HAVE_MEMSET == 0 */

#if HAVE_STRLEN == 0
/*
 * Return the length in characters of STR
 */
int	strlen(const char *str)
{
  char	*str_p = str;
  int	len;
  
  for (len = 0; *str_p != '\0'; len++) {
    str_p++;
  }
  
  return len;
}
#endif /* HAVE_STRLEN == 0 */
