/*
 * Diskheap OS defines...
 *
 * Copyright 2002 by Gray Watson
 *
 * This file is part of the diskheap package.
 *
 * Permission to use, copy, modify, and distribute this software for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of Gray Watson not be used in advertising
 * or publicity pertaining to distribution of the document or software
 * without specific, written prior permission.
 *
 * Gray Watson makes no representations about the suitability of the
 * software described herein for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * The author may be contacted via http://256.com/gray/
 *
 * $Id: diskheap_os.h,v 1.9 2002/02/19 15:32:34 gray Exp $
 */

#ifndef __DISKHEAP_OS_H__
#define __DISKHEAP_OS_H__

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

/*<<<<<<<<<<  The below prototypes are auto-generated by fillproto */

/*
 * int _dh_os_open
 *
 * DESCRIPTION:
 *
 * Perform an open call.  Should not be necessary except for crappy
 * implementation of 64-bit filesystems.
 *
 * RETURNS:
 *
 * Success - File description >= 0.
 *
 * Failure - < 0
 *
 * ARGUMENTS:
 *
 * file -> Name of the file we are opening.
 *
 * create_b -> 1 if we are creating the file otherwise 0 if we are
 * just opening it.
 *
 * read_only_b -> 1 if we are opening in read-only mode.  Ignored if
 * creating.
 *
 * mode -> The mode to create the file if we are creating.
 */
extern
int	_dh_os_open(const char *file, const int create_b,
		    const int read_only_b, const int mode);

/*
 * void _dh_os_close
 *
 * DESCRIPTION:
 *
 * Close a file descriptor.
 *
 * RETURNS:
 *
 * None.
 *
 * ARGUMENTS:
 *
 * fd -> File descriptor to close.
 */
extern
void	_dh_os_close(const int fd);

/*
 * int _dh_os_pread
 *
 * DESCRIPTION:
 *
 * Perform a pread using 64-bit (if available) information.
 *
 * RETURNS:
 *
 * Success - DISKHEAP_ERROR_NONE
 *
 * Failure - DISKHEAP error code
 *
 * ARGUMENTS:
 *
 * dh_p <-> Disk pointer we are reading from.
 *
 * buf -> Buffer into which we are reading.
 *
 * bytes_n -> Number of bytes to be read.
 *
 * block_num -> Number of the block where we are reading.
 *
 * offset -> Offset in the block where we are reading.  This can also
 * be used as a 32-bit offset if the block-number if 0.
 */
extern
int	_dh_os_pread(diskheap_t *dh_p, void *buf, const unsigned int bytes_n,
		     const unsigned int block_num, const unsigned int offset);

/*
 * int _dh_os_pwrite
 *
 * DESCRIPTION:
 *
 * Perform a pwrite using 64-bit (if available) information.
 *
 * RETURNS:
 *
 * Success - DISKHEAP_ERROR_NONE
 *
 * Failure - DISKHEAP error code
 *
 * ARGUMENTS:
 *
 * dh_p <-> Disk pointer we are reading from.
 *
 * buf -> Buffer from which we are writing.
 *
 * bytes_n -> Number of bytes to be writing.
 *
 * block_num -> Number of the block where we are writing.
 *
 * offset -> Offset in the block where we are writing.
 */
extern
int	_dh_os_pwrite(diskheap_t *dh_p, const void *buf,
		      const unsigned int bytes_n, const unsigned int block_num,
		      const unsigned int offset);

/*
 * int _dh_os_srandom
 *
 * DESCRIPTION:
 *
 * Seed the random number generator.
 *
 * RETURNS:
 *
 * The seed chosen by the routine.
 *
 * ARGUMENTS:
 *
 * seed -> Seed specified by the user.  0 for none.
 */
extern
int	_dh_os_srandom(const int user_seed);

/*
 * int _dh_os_random
 *
 * DESCRIPTION:
 *
 * Get a random number.
 *
 * RETURNS:
 *
 * A pseudo-random number between 0 and 2^31-1.
 *
 * ARGUMENTS:
 *
 * None.
 */
extern
int	_dh_os_random(void);

/*
 * unsigned int _dh_os_time
 *
 * DESCRIPTION:
 *
 * Get the time in seconds.
 *
 * RETURNS:
 *
 * The time.
 *
 * ARGUMENTS:
 *
 * None.
 */
extern
unsigned int	_dh_os_time(void);

/*
 * int _dh_os_fsync
 *
 * DESCRIPTION:
 *
 * Sync the file descriptor if fsync is enabled by the system.
 *
 * RETURNS:
 *
 * Success - DISKHEAP_ERROR_NONE
 *
 * Failure - DISKHEAP error code
 *
 * ARGUMENTS:
 *
 * fd -> File descriptor we are syncing.
 */
extern
int	_dh_os_fsync(const int fd);

/*<<<<<<<<<<   This is end of the auto-generated output from fillproto. */

#ifdef __cplusplus
}
#endif


#endif /* ! __DISKHEAP_OS_H__ */
